@@ -36,13 +36,13 @@ appropriately.
 See https://github.com/tcalmant/jsonrpclib for more info.
 
 :authors: Josh Marshall, Thomas Calmant
-:copyright: Copyright 2018, Thomas Calmant
+:copyright: Copyright 2019, Thomas Calmant
 :license: Apache License 2.0
-:version: 0.3.2
+:version: 0.4.0
 
 ..
 
-    Copyright 2018 Thomas Calmant
+    Copyright 2019 Thomas Calmant
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -60,12 +60,15 @@ See https://github.com/tcalmant/jsonrpclib for more info.
 # Standard library
 import contextlib
 import logging
+import os
+import socket
 import sys
 import uuid
 
 try:
     # Python 3
     # pylint: disable=F0401,E0611
+    from http.client import HTTPConnection
     from urllib.parse import splittype, splithost
     from xmlrpc.client import Transport as XMLTransport
     from xmlrpc.client import SafeTransport as XMLSafeTransport
@@ -74,6 +77,7 @@ try:
 except ImportError:
     # Python 2
     # pylint: disable=F0401,E0611
+    from httplib import HTTPConnection
     from urllib import splittype, splithost
     from xmlrpclib import Transport as XMLTransport
     from xmlrpclib import SafeTransport as XMLSafeTransport
@@ -96,7 +100,7 @@ import jsonrpclib.utils as utils
 # ------------------------------------------------------------------------------
 
 # Module version
-__version_info__ = (0, 3, 2)
+__version_info__ = (0, 4, 0)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -461,7 +465,79 @@ class SafeTransport(TransportMixIn, XMLSafeTransport):
     """
     def __init__(self, config, context):
         TransportMixIn.__init__(self, config, context)
-        XMLSafeTransport.__init__(self)
+        try:
+            # Give the context to XMLSafeTransport, to avoid it setting the
+            # context to None.
+            # See https://github.com/tcalmant/jsonrpclib/issues/39
+            XMLSafeTransport.__init__(self, context=context)
+        except TypeError:
+            # On old versions of Python (Pre-2014), the context argument
+            # wasn't available
+            XMLSafeTransport.__init__(self)
+
+# ------------------------------------------------------------------------------
+
+
+class UnixHTTPConnection(HTTPConnection):
+    """
+    Replaces the connect() method of HTTPConnection to use a Unix socket
+    """
+    def __init__(self, path, *args, **kwargs):
+        """
+        Constructs the HTTP connection.
+
+        Forwards all given arguments except ``path`` to the constructor of
+        HTTPConnection
+
+        :param path: Path to the Unix socket
+        """
+        HTTPConnection.__init__(self, path, *args, **kwargs)
+        self.path = path
+
+    def connect(self):
+        """
+        Connects to the described server
+        """
+        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        self.sock.connect(self.path)
+
+
+class UnixTransport(TransportMixIn, XMLTransport):
+    """
+    Mixed-in HTTP transport over a UNIX socket
+    """
+    def __init__(self, config, path=None):
+        """
+        :param config: The jsonrpclib configuration
+        :param path: Path to the Unix socket (overrides the host name later)
+        """
+        TransportMixIn.__init__(self, config)
+        XMLTransport.__init__(self)
+        # Keep track of the given path, if any
+        self.__unix_path = os.path.abspath(path) if path else None
+
+    def make_connection(self, host):
+        """
+        Connect to server.
+
+        Return an existing connection if possible.
+        This allows HTTP/1.1 keep-alive.
+
+        Code copied from xmlrpc.client (Python 3)
+
+        :param host: Target host (ignored if a path was given)
+        :return A UnixHTTPConnection object
+        """
+        if self.__unix_path:
+            host = self.__unix_path
+
+        if self._connection and host == self._connection[0]:
+            return self._connection[1]
+
+        # create a HTTP connection object from a host descriptor
+        path, self._extra_headers, _ = self.get_host_info(host)
+        self._connection = host, UnixHTTPConnection(path)
+        return self._connection[1]
 
 # ------------------------------------------------------------------------------
 
@@ -492,21 +568,43 @@ class ServerProxy(XMLServerProxy):
         self.__version = version or config.version
 
         schema, uri = splittype(uri)
+        use_unix = False
+        if schema.startswith("unix+"):
+            schema = schema[len("unix+"):]
+            use_unix = True
+
         if schema not in ('http', 'https'):
             _logger.error("jsonrpclib only support http(s) URIs, not %s",
                           schema)
             raise IOError('Unsupported JSON-RPC protocol.')
 
         self.__host, self.__handler = splithost(uri)
-        if not self.__handler:
+        if use_unix:
+            unix_path = self.__handler
+            self.__handler = '/'
+        elif not self.__handler:
             # Not sure if this is in the JSON spec?
             self.__handler = '/'
 
         if transport is None:
-            if schema == 'https':
+            if use_unix:
+                if schema == "http":
+                    # In Unix mode, we use the path part of the URL (handler)
+                    # as the path to the socket file
+                    transport = UnixTransport(
+                        config=config, path=unix_path
+                    )
+            elif schema == 'https':
                 transport = SafeTransport(config=config, context=context)
             else:
                 transport = Transport(config=config)
+
+            if transport is None:
+                raise IOError(
+                    "Unhandled combination: UNIX={}, protocol={}"
+                    .format(use_unix, schema)
+                )
+
         self.__transport = transport
 
         self.__encoding = encoding
@@ -787,7 +885,10 @@ class MultiCallIterator(object):
         """
         for item in self.results:
             yield self.__get_result(item)
-        raise StopIteration
+
+        # Since Python 3.7, we must return instead of raising a StopIteration
+        # (see PEP-479)
+        return
 
     def __getitem__(self, i):
         """
